@@ -1,6 +1,4 @@
-from typing import Any, cast
-
-from gitlab.base import RESTManager, RESTObject
+from gitlab.base import RESTObject
 from gitlab.mixins import CreateMixin, DownloadMixin, GetWithoutIdMixin, RefreshMixin
 from gitlab.types import RequiredOptional
 
@@ -20,50 +18,40 @@ class GroupExport(DownloadMixin, RESTObject):
     _id_attr = None
 
 
-class GroupExportManager(GetWithoutIdMixin, CreateMixin, RESTManager):
+class GroupExportManager(GetWithoutIdMixin[GroupExport], CreateMixin[GroupExport]):
     _path = "/groups/{group_id}/export"
     _obj_cls = GroupExport
     _from_parent_attrs = {"group_id": "id"}
 
-    def get(self, **kwargs: Any) -> GroupExport:
-        return cast(GroupExport, super().get(**kwargs))
-
 
 class GroupImport(RESTObject):
     _id_attr = None
 
 
-class GroupImportManager(GetWithoutIdMixin, RESTManager):
+class GroupImportManager(GetWithoutIdMixin[GroupImport]):
     _path = "/groups/{group_id}/import"
     _obj_cls = GroupImport
     _from_parent_attrs = {"group_id": "id"}
 
-    def get(self, **kwargs: Any) -> GroupImport:
-        return cast(GroupImport, super().get(**kwargs))
-
 
 class ProjectExport(DownloadMixin, RefreshMixin, RESTObject):
     _id_attr = None
 
 
-class ProjectExportManager(GetWithoutIdMixin, CreateMixin, RESTManager):
+class ProjectExportManager(
+    GetWithoutIdMixin[ProjectExport], CreateMixin[ProjectExport]
+):
     _path = "/projects/{project_id}/export"
     _obj_cls = ProjectExport
     _from_parent_attrs = {"project_id": "id"}
     _create_attrs = RequiredOptional(optional=("description",))
 
-    def get(self, **kwargs: Any) -> ProjectExport:
-        return cast(ProjectExport, super().get(**kwargs))
-
 
 class ProjectImport(RefreshMixin, RESTObject):
     _id_attr = None
 
 
-class ProjectImportManager(GetWithoutIdMixin, RESTManager):
+class ProjectImportManager(GetWithoutIdMixin[ProjectImport]):
     _path = "/projects/{project_id}/import"
     _obj_cls = ProjectImport
     _from_parent_attrs = {"project_id": "id"}
-
-    def get(self, **kwargs: Any) -> ProjectImport:
-        return cast(ProjectImport, super().get(**kwargs))
