@@ -15,19 +15,20 @@ def test_epics(group):
     assert group.epics.list()
 
 
-@pytest.mark.xfail(reason="404 on issue.id")
 def test_epic_issues(epic, issue):
     assert not epic.issues.list()
 
+    # FYI: Creating an issue causes a note to be created
     epic_issue = epic.issues.create({"issue_id": issue.id})
     assert epic.issues.list()
 
+    # FYI: Deleting an issue causes a note to be created
     epic_issue.delete()
-    assert not epic.issues.list()
 
 
 def test_epic_notes(epic):
-    assert not epic.notes.list()
+    notes = epic.notes.list(get_all=True)
 
     epic.notes.create({"body": "Test note"})
-    assert epic.notes.list()
+    new_notes = epic.notes.list(get_all=True)
+    assert len(new_notes) == (len(notes) + 1), f"{new_notes} {notes}"
