@@ -1,6 +1,5 @@
 import base64
 import os
-import sys
 import tarfile
 import time
 import zipfile
@@ -74,9 +73,6 @@ def test_repository_archive(project):
     assert archive == archive2
 
 
-# NOTE(jlvillal): Support for using tarfile.is_tarfile() on a file or file-like object
-# was added in Python 3.9
-@pytest.mark.skipif(sys.version_info < (3, 9), reason="requires python3.9 or higher")
 @pytest.mark.parametrize(
     "format,assertion",
     [
@@ -165,6 +161,23 @@ def test_commit_discussion(project):
     note_from_get.delete()
 
 
+def test_cherry_pick_commit(project):
+    commits = project.commits.list()
+    commit = commits[1]
+    parent_commit = commit.parent_ids[0]
+
+    # create a branch to cherry pick onto
+    project.branches.create({"branch": "test", "ref": parent_commit})
+    cherry_pick_commit = commit.cherry_pick(branch="test")
+
+    expected_message = f"{commit.message}\n\n(cherry picked from commit {commit.id})"
+    assert cherry_pick_commit["message"].startswith(expected_message)
+
+    with pytest.raises(gitlab.GitlabCherryPickError):
+        # Two cherry pick attempts should raise GitlabCherryPickError
+        commit.cherry_pick(branch="test")
+
+
 def test_revert_commit(project):
     commit = project.commits.list()[0]
     revert_commit = commit.revert(branch="main")
