@@ -25,7 +25,7 @@ def test_all_v4_objects_are_imported() -> None:
     assert len(gitlab.v4.objects.__path__) == 1
 
     init_files: Set[str] = set()
-    with open(gitlab.v4.objects.__file__, "r", encoding="utf-8") as in_file:
+    with open(gitlab.v4.objects.__file__, encoding="utf-8") as in_file:
         for line in in_file.readlines():
             if line.startswith("from ."):
                 init_files.add(line.rstrip())
